package at.gv.egiz.moazs.util;

import at.gv.egiz.moazs.repository.DeliveryRepository;
import at.gv.egiz.moazs.scheme.SOAPUtils;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.function.Function;

import static at.gv.egiz.moazs.MoaZSException.moaZSException;

@Component
public class StoreSOAPBodyBinaryInRepositoryInterceptor extends AbstractPhaseInterceptor<Message> {

    private static final Logger log = LoggerFactory.getLogger(StoreSOAPBodyBinaryInRepositoryInterceptor.class);

    private final CXFMessageUtils messageUtils;
    private final SOAPUtils soapUtils;
    private final Map<String, Function<String, String>> idGenerators;
    private final DeliveryRepository repository;

    @Autowired
    public StoreSOAPBodyBinaryInRepositoryInterceptor(CXFMessageUtils extractor, SOAPUtils soapUtils,
                                                      Map<String, Function<String, String>> idGenerators,
                                                      DeliveryRepository repository) {
        super(Phase.RECEIVE);
        this.messageUtils = extractor;
        this.soapUtils = soapUtils;
        this.idGenerators = idGenerators;
        this.repository = repository;
    }

    public void handleMessage(Message message) {

        try {
            byte[] content = messageUtils.copyContent(message);

            if (content.length <= 0) {
                return;
            }

            Element document = soapUtils.toDOM(content);
            var rootTagNode = soapUtils.getChildElementOfSoapBody(document);
            var rootTagLocalName = rootTagNode.getLocalName();

            if (!idGenerators.containsKey(rootTagLocalName)) {
                log.trace("Child element {} of <soap:Body> is unknown. Will not store message.", rootTagLocalName);
                return;
            }

            String appDeliveryID = soapUtils.getAppDeliveryIDFrom(document);
            var id = idGenerators.get(rootTagLocalName).apply(appDeliveryID);
            byte[] response = soapUtils.toBytes(rootTagNode);
            repository.store(id, response);

            if(log.isTraceEnabled()) {
                log.trace("Msg Response with AppDeliveryID={} unwrapped and stored: {}. ",
                        appDeliveryID, new String(response, StandardCharsets.UTF_8));
            }

        } catch (ParserConfigurationException | SAXException | IOException | NullPointerException | TransformerException e) {
            throw moaZSException("Could not extract signed data from message.", e);
        }
    }

}
